cordova.define("ntaapp-plugin-jpki.JPKIPlugin", function(require, exports, module) {
var JPKIPlugin = {
    Sign: function (password, data) {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.Sign');
            cordova.exec((ret) => {
                let signatureString;
                let platformName = window.device.platform;

                // Big Sur向けに修正 2021/10/22
                if (platformName == "Mac OS X" || platformName == "macOS") {
                    let tmp = [];
                    let len = 1024;
                    for (let p = 0; p < ret.byteLength; p += len) {
                      tmp.push(String.fromCharCode.apply("", new Uint8Array(ret.slice(p, p + len))));
                    }
                    signatureString = btoa(tmp.join(""));
                } else if(platformName == "Android"){
                    signatureString = btoa(ret);
                } else if(platformName == "windows"){
                    let tmp = [];
                    let len = 1024;
                    for (let p = 0; p < ret.byteLength; p += len) {
                      tmp.push(String.fromCharCode.apply("", new Uint8Array(ret.slice(p, p + len))));
                    }
                    signatureString = btoa(tmp.join(""));
                } else if(platformName == "iOS"){
                    signatureString = ret;
                }

                resolve(signatureString);
            }, (err) => {
                reject(getJpkiErrorCode(err));                
            }, 'JPKIPlugin', 'Sign', [password, data]);
        }));
    },

    SignMultiple: function (password, dataArray) {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.SignMultiple');
            cordova.exec((retArray) => {
                let signatureStrings = [];
                
                for (let ret of retArray) {
                    signatureStrings.push(convertToBase64(ret));
                }

                resolve(signatureStrings);
            }, (err) => {
                reject(getJpkiErrorCode(err));
            }, 'JPKIPlugin', 'SignMultiple', [password, dataArray]);
        }));
    },

    Verify: function (password, data, sign, certificate) {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.Verify');
            let signBase64 = sign;
            cordova.exec( (ret) => {
                resolve(0);
            }, (err) => {
                reject(getJpkiErrorCode(err));
            }, 'JPKIPlugin', 'Verify', [password, data, signBase64, certificate]);
        }));
    },

    VerifyMultiple: function (password, dataArray, signArray) {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.VerifyMultiple');
            let signBase64Array = signArray;
            cordova.exec( (retArray) => {
                resolve(0);
            }, (err) => {
                reject(getJpkiErrorCode(err));
            }, 'JPKIPlugin', 'VerifyMultiple', [password, dataArray, signBase64Array]);
        }));
    },

    Certificate: function (password) {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.Certificate');
            cordova.exec( (ret) => {
                let certificateString;
                let platformName = window.device.platform;
                
                // Big Sur向けに修正 2021/10/22
                if (platformName == "windows" || platformName == "Mac OS X" ||
                platformName == "macOS" ) {
                    let tmp = [];
                    let len = 1024;
                    for (let p = 0; p < ret.byteLength; p += len) {
                      tmp.push(String.fromCharCode.apply("", new Uint8Array(ret.slice(p, p + len))));
                    }
                    certificateString = btoa(tmp.join(""));
                } else if(platformName == "Android"){
                    certificateString = btoa(ret);
                } else if(platformName == "iOS") {
                    certificateString = ret;
                }
                
                resolve(certificateString);
            }, (err) => {
                reject(getJpkiErrorCode(err));
            },  'JPKIPlugin', 'Certificate', [password]);
        }));
    },

    Initialize: function () {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.Initialize');
            cordova.exec( resolve, (err) => {
                reject(getJpkiErrorCode(err));
            }, 'JPKIPlugin', 'Initialize', []);
        }));
    },

    Finalize: function () {
        return addJpkiQueue(() => new Promise((resolve, reject) => {
            console.log('JPKIPlugin.Finalize');
            cordova.exec( resolve, (err) => {
                reject(getJpkiErrorCode(err));
            }, 'JPKIPlugin', 'Finalize', []);
        }));
    }
};

let JpkiPromiseQueue = Promise.resolve();
function addJpkiQueue(func) {
    JpkiPromiseQueue = JpkiPromiseQueue.then(func);
    let promise = JpkiPromiseQueue;
    JpkiPromiseQueue = JpkiPromiseQueue.catch(reason => console.log(reason));
    return promise;
}

function convertToBase64(value) {
    let base64Str = '';
    let isArrayBuffer = value instanceof ArrayBuffer;
    let isBase64 = !isArrayBuffer && atob(value) != null;
    let isMaybeBinaryString = !isBase64 && typeof value === 'string';

    if (isArrayBuffer) {
        let tmp = [];
        let len = 1024;
        for (let p = 0; p < value.byteLength; p += len) {
            tmp.push(String.fromCharCode.apply('', new Uint8Array(value.slice(p, p + len))));
        }
        base64Str = btoa(tmp.join(''));
    } else if (isBase64) {
        base64Str = value;
    } else if (isMaybeBinaryString) {
        base64Str = btoa(value);
    }
    return base64Str;
}

function getJpkiErrorCode(code) {
    console.log("JpkiErrorCode: " + code);
    let fixedCode = -999;
    try {
        if(typeof code == "number") {
            fixedCode = code;
        } else if(typeof code == "string") {
            fixedCode = parseInt(code);
        } 
    } catch(error) {
        console.error("error=" + error);
    }
    console.log("JpkiErrorCode(fixed): " + fixedCode);
    return fixedCode;
}

//htmlはこのオブジェクトを呼ぶ
module.exports = JPKIPlugin;

});
